@@ -4,7 +4,6 @@
  */
 
 #include "homescreenutils.h"
-#include "applicationlistmodel.h"
 #include "favoritesmodel.h"
 
 #include <QtQml>
@@ -19,20 +18,6 @@ HomeScreenUtils::HomeScreenUtils(QObject *parent)
 
 HomeScreenUtils::~HomeScreenUtils() = default;
 
-ApplicationListModel *HomeScreenUtils::applicationListModel()
-{
-    if (!m_applicationListModel) {
-        if (m_showAllApps) {
-            m_applicationListModel = new ApplicationListModel(this);
-        } else {
-            m_applicationListModel = new FavoritesModel(this);
-        }
-      //  m_applicationListModel->setApplet(this);
-        m_applicationListModel->loadApplications();
-    }
-    return m_applicationListModel;
-}
-
 void HomeScreenUtils::stackBefore(QQuickItem *item1, QQuickItem *item2)
 {
     if (!item1 || !item2 || item1 == item2 || item1->parentItem() != item2->parentItem()) {
