@@ -12,6 +12,7 @@
 #include <QtQml>
 
 #include <KWayland/Client/connection_thread.h>
+#include <KWayland/Client/output.h>
 #include <KWayland/Client/plasmashell.h>
 #include <KWayland/Client/plasmawindowmanagement.h>
 #include <KWayland/Client/registry.h>
@@ -23,12 +24,83 @@
 static const QString s_kwinService = QStringLiteral("org.kde.KWin");
 constexpr int ACTIVE_WINDOW_UPDATE_INVERVAL = 250;
 
+class OutputsModel : public QAbstractListModel
+{
+public:
+    enum Roles {
+        Model = Qt::DisplayRole,
+        Geometry = Qt::UserRole,
+        Position,
+        Output,
+    };
+
+    OutputsModel(QObject *parent)
+        : QAbstractListModel(parent)
+    {
+    }
+
+    QHash<int, QByteArray> roleNames() const override
+    {
+        return {
+            {Model, "modelName"},
+            {Geometry, "geometry"},
+            {Position, "position"},
+            {Output, "output"},
+        };
+    }
+
+    void createOutput(wl_output *output)
+    {
+        auto newOutput = new KWayland::Client::Output(this);
+        connect(newOutput, &KWayland::Client::Output::removed, this, [this, newOutput] {
+            auto i = m_outputs.indexOf(newOutput);
+            Q_ASSERT(i >= 0);
+            beginRemoveRows({}, i, i);
+            m_outputs.removeAt(i);
+            endRemoveRows();
+        });
+        newOutput->setup(output);
+        beginInsertRows({}, m_outputs.count(), m_outputs.count());
+        m_outputs.append(newOutput);
+        endInsertRows();
+    }
+
+    int rowCount(const QModelIndex &parent) const override
+    {
+        return parent.isValid() ? 0 : m_outputs.count();
+    }
+
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override
+    {
+        if (index.row() >= m_outputs.count()) {
+            return {};
+        }
+
+        auto o = m_outputs[index.row()];
+        switch (role) {
+        case Model:
+            return o->model();
+        case Geometry:
+            return o->geometry();
+        case Position:
+            return o->globalPosition();
+        case Output:
+            return QVariant::fromValue<QObject *>(o);
+        }
+        return {};
+    }
+
+private:
+    QVector<KWayland::Client::Output *> m_outputs;
+};
+
 // helper class to expose the NOTIFY in the properties
 class KwinVirtualKeyboardInterface : public OrgKdeKwinVirtualKeyboardInterface
 {
     Q_OBJECT
     Q_PROPERTY(bool active READ active WRITE setActive NOTIFY activeChanged)
     Q_PROPERTY(bool enabled READ enabled WRITE setEnabled NOTIFY enabledChanged)
+    Q_PROPERTY(bool visible READ visible NOTIFY visibleChanged)
 public:
     KwinVirtualKeyboardInterface()
         : OrgKdeKwinVirtualKeyboardInterface(QStringLiteral("org.kde.KWin"), QStringLiteral("/VirtualKeyboard"), QDBusConnection::sessionBus())
@@ -40,6 +112,7 @@ TaskPanel::TaskPanel(QObject *parent, const QVariantList &args)
     : Plasma::Containment(parent, args)
     , m_showingDesktop(false)
     , m_windowManagement(nullptr)
+    , m_outputsModel(new OutputsModel(this))
 {
     setHasConfigurationInterface(true);
     m_activeTimer = new QTimer(this);
@@ -48,6 +121,8 @@ TaskPanel::TaskPanel(QObject *parent, const QVariantList &args)
     connect(m_activeTimer, &QTimer::timeout, this, &TaskPanel::updateActiveWindow);
     initWayland();
 
+    qmlRegisterUncreatableType<KWayland::Client::Output>("org.kde.plasma.phone.taskpanel", 1, 0, "Output", "nope");
+    qmlRegisterUncreatableType<OutputsModel>("org.kde.plasma.phone.taskpanel", 1, 0, "OutputsModel", "nope");
     qmlRegisterSingletonType<OrgKdeKwinVirtualKeyboardInterface>("org.kde.plasma.phone.taskpanel",
                                                                  1,
                                                                  0,
@@ -55,6 +130,12 @@ TaskPanel::TaskPanel(QObject *parent, const QVariantList &args)
                                                                  [](QQmlEngine *, QJSEngine *) -> QObject * {
                                                                      return new KwinVirtualKeyboardInterface;
                                                                  });
+
+    connect(this, &Plasma::Containment::locationChanged, this, &TaskPanel::locationChanged);
+    connect(this, &Plasma::Containment::locationChanged, this, [this] {
+        auto l = location();
+        setFormFactor(l == Plasma::Types::LeftEdge || l == Plasma::Types::RightEdge ? Plasma::Types::Vertical : Plasma::Types::Horizontal);
+    });
 }
 
 TaskPanel::~TaskPanel() = default;
@@ -80,6 +161,9 @@ void TaskPanel::initWayland()
     }
     auto *registry = new Registry(this);
     registry->create(connection);
+    connect(registry, &Registry::outputAnnounced, m_outputsModel, [this, registry](quint32 name, quint32 version) {
+        m_outputsModel->createOutput(registry->bindOutput(name, version));
+    });
     connect(registry, &Registry::plasmaWindowManagementAnnounced, this, [this, registry](quint32 name, quint32 version) {
         m_windowManagement = registry->createPlasmaWindowManagement(name, version, this);
         qRegisterMetaType<QVector<int>>("QVector<int>");
@@ -206,6 +290,21 @@ void TaskPanel::closeActiveWindow()
     }
 }
 
-K_EXPORT_PLASMA_APPLET_WITH_JSON(taskpanel, TaskPanel, "metadata.json")
+void TaskPanel::sendWindowToOutput(const QString &uuid, KWayland::Client::Output *output)
+{
+    const auto windows = m_windowManagement->windows();
+    for (auto w : windows) {
+        if (w->uuid() == uuid) {
+            w->sendToOutput(output);
+        }
+    }
+}
+
+QAbstractItemModel *TaskPanel::outputs() const
+{
+    return m_outputsModel;
+}
+
+K_PLUGIN_CLASS_WITH_JSON(TaskPanel, "metadata.json")
 
 #include "taskpanel.moc"
